/* this file is only needed, if the spl functions are not inline expanded.
   It's a bit easier to fiddle with spl-settings changing one file than 
   recompiling the whole kernel.. */

#ifdef SPL_DEBUG

#include <machine/psl.h>

#define PRINTF(a) if (0) printf(a)

/* Set the interrupt priority level and save the old level */
int spln(unsigned short spl)
{
  register unsigned short sr;
  register int scratch;

  asm volatile ("movew sr,%0;"
		"movew %0,%1;"
		"andiw #0xf8ff,%1;"
		"orw %2,%1;"
		"movew %1,sr;"
		: "=d" (sr), "=d" (scratch) : "d" (spl) : "cc");
  return(sr & 0x700);
}


/* Set the interrupt priority level back to the previous level */
void splx(unsigned short pri)
{
#if 0
  printf ("[splx%d]", pri);
#endif
  spln(pri);
}


/* efficiency is secondary for this file.... */

#define LEVEL(num) int spl##num(void) { return spln(PSL_IPL##num); }

LEVEL(0)
LEVEL(1)
LEVEL(2)
LEVEL(3)
LEVEL(4)
LEVEL(5)
LEVEL(6)
LEVEL(7)

/* block software timeouts */
int splsoftclock() 
{ PRINTF ("[splsoftclock]"); return spl1(); }

/* block SCSI interrupts (logically spl2, but must match at least spltty) */
int splbio()       
{ PRINTF ("[splbio]"); return spltty(); }

/* block hardware clock interrupts (CIAB generates INT6) */
int splclock()     
{ PRINTF ("[splclock]"); return spl6(); }

/* block serial line and keyboard interrupts */
int spltty()       
{ PRINTF ("[spltty]"); return spl5(); }

/* block network interrupts (at least SLIP, ie. spltty, if nothing else) */
int splimp()       
{ PRINTF ("[splimp]"); return spltty(); }

/* block ALL io devices (still hardcoded in ds_routines.c to splsched!!!) */
int splio()        
{ PRINTF ("[splio]"); return spl5(); }

/* block all interrupts that may affect scheduling (eg. switch tasks) */
int splsched()     
{ PRINTF ("[splsched]"); return spl7(); }

/* not used, what was the purpose for this level? */
int splvm()        
{ PRINTF ("[splvm]"); return spl7(); }

/* block all interrupts */
int splhigh()      
{ PRINTF ("[splhigh]"); return spl7(); }

#endif /* SPL_DEBUG */
